package org.eclipse.epsilon.egl.dom.ast2dom.workbench;


import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.URL;

import org.antlr.runtime.ANTLRInputStream;
import org.antlr.runtime.CommonTokenStream;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.eclipse.epsilon.commons.parse.AST;
import org.eclipse.epsilon.commons.parse.EpsilonTreeAdaptor;
import org.eclipse.epsilon.egl.dom.ast2dom.EGLAst2DomContext;
import org.eclipse.epsilon.egl.dom.printer.EGLDomElementPrinterFactory;
import org.eclipse.epsilon.egl.parse.EglLexer;
import org.eclipse.epsilon.egl.parse.EglParser;
import org.eclipse.epsilon.egl.preprocessor.Preprocessor;
import org.eclipse.epsilon.eol.dom.DomElement;
import org.eclipse.epsilon.eol.dom.Program;
import org.eclipse.epsilon.eol.dom.ast2dom.Ast2DomContext;
import org.eclipse.epsilon.eol.parse.EolLexer;
import org.eclipse.epsilon.eol.parse.EolParser;

public class Ast2DomWorkbench {

	public static void main(String[] args) throws Exception {
		new Ast2DomWorkbench().run();
	}
	
	public void run() throws Exception {
		
		String egl_script = "test.egl";
		String dom_model = "model/text.xmi";
		DomElement de;
		
		URL url = getClass().getResource(egl_script);
		File file = new File(url.getPath());

		// parse EGL template
		EglLexer egl_lexer = new EglLexer(new FileInputStream(file));
		EpsilonTreeAdaptor adaptor = new EpsilonTreeAdaptor(file);
		EglParser egl_parser = new EglParser(egl_lexer, adaptor);
		egl_parser.parse();

		// convert to EOL
		Preprocessor pp = new Preprocessor();
		String egl_ast = pp.convertToEol(egl_parser.getAST());

		// 'normal' EOL parsing
		ANTLRInputStream input = new ANTLRInputStream(new ByteArrayInputStream(egl_ast.getBytes()));
		EolLexer lexer = new EolLexer(input);
		CommonTokenStream stream = new CommonTokenStream(lexer);
		EolParser parser = new EolParser(stream);
		parser.setDeepTreeAdaptor(adaptor);

		// get AST
		AST ast = (AST) parser.eolModule().getTree();		
		System.err.println(ast.toStringTree());

		// create DOM elements from AST
		de = createDom(ast);

		// marshalling
		saveDom(dom_model, de) ;
		
		// unmarshalling
		de = loadDom(dom_model);
		
		// create EGL from DOM
		String egl_code = createEgl(de, false);

		// print EGL
		System.out.println(egl_code);

	}
	
	private DomElement createDom(AST ast) {
		Ast2DomContext context = new EGLAst2DomContext();
		DomElement de = context.getDomElementCreatorFactory().createDomElement(ast, null, context);
		return de;
	}
	
	private void saveDom(String dom_model, DomElement de) throws IOException {
		ResourceSet resourceSet = new ResourceSetImpl();
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put("xmi", new XMIResourceFactoryImpl());
		Resource resource = resourceSet.createResource(URI.createFileURI(new File(dom_model).getAbsolutePath()));
		resource.getContents().add(de);
		resource.save(null);
	}
	
	private DomElement loadDom(String dom_model) {
		// TODO use internal methods?
		ResourceSet resourceSet = new ResourceSetImpl();
		
		// load EOL model into registry
		resourceSet.getPackageRegistry().put(org.eclipse.epsilon.eol.dom.DomPackage.eNS_URI, org.eclipse.epsilon.eol.dom.DomPackage.eINSTANCE);
		// load EGL model into registry
		resourceSet.getPackageRegistry().put(org.eclipse.epsilon.egl.dom.DomPackage.eNS_URI, org.eclipse.epsilon.egl.dom.DomPackage.eINSTANCE);

		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put("xmi", new XMIResourceFactoryImpl());
		Resource resource = resourceSet.getResource(URI.createURI(dom_model), true);
		return (DomElement)resource.getContents().get(0);
	}
	
	private String createEgl(DomElement de, Boolean brackets) {
		String egl = new EGLDomElementPrinterFactory().print((Program)de);
		if (brackets == true)
			egl = "[%" + egl + "%]";
		return egl;
	}
	
}
